/* In our grand finale project we decided to tackle the biggest issue students face while trying to ace, or pass, their exams period. 
 *  We offer 6 main features that will boost the overall studying performance of our users:
 *  1) Light detector - which will notify you via Blynk when your study environment is too dark
 *  
 *  2) Sound detector - which will notify you via Blynk when your study environment is too noisy
 *  
 *  3) Study timer - start a timer for study time and will be notified when it's over and you're free
 *  
 *  4) Break state - when you go on a break the study buddy will go on a break too
 *  
 *  5) Distance detector - which will notify you via Integromat when you try to escape and drift for more than 100m from your study zone.
 *  
 *  6) Phone lift detector - which will make every phone use during study time a nightmare.
 *  
 *  We've blended between Integromat, Blynk, Circuit Playground Express services in the creation of this project, trying to address
 *  the major problems a student faces during their study efforts. 
 */

//BLINK CONNECTION

#define BLYNK_PRINT SerialUSB
#define BLYNK_DEBUG SerialUSB

#include <Wire.h>
#include <SPI.h> 
#include <ESP8266_Lib.h>
#include <BlynkSimpleShieldEsp8266.h>
#include <Adafruit_CircuitPlayground.h>

char auth[] = "moipAMfw2LsTiYoOwnc6F95HGNo7xxco";
char ssid[] = "Baroni";
char pass[] = "0545786935";

#define NOTE_C4  262
#define EspSerial Serial1
#define ESP8266_BAUD 9600
ESP8266 wifi(&EspSerial);

//////////////////////****INITIALIZATION****///////////////////////////////

void initialize(){
  Blynk.virtualWrite(V0, "Get a quite place to study at!");
  Blynk.virtualWrite(V1, "");
  Blynk.virtualWrite(V2, "Get a lit place to study at!");
  Blynk.virtualWrite(V3, "");
  Blynk.virtualWrite(V5, "");
  for (int i = 0; i < 10; ++i) {
      CircuitPlayground.setPixelColor(i, 0, 0, 0);
  }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////

float soundSensor;
float lightSensor;
boolean noteNoise = false;
boolean noteDark = false;

//////////////////////****LEARNING_TIMER****///////////////////////////////

int tenprecent;
boolean isTimer = false;
int timerStart; //Checks if the start timer button of the blynk was pressed
unsigned long curTime;
unsigned long timeToLearn; //The time in seconds that the user wants the timer to be
boolean timerOn(){
    Blynk.virtualWrite(V1, (timeToLearn - curTime));
    Blynk.virtualWrite(V3, (timeToLearn - curTime));
    Blynk.virtualWrite(V2, "Timer on");
    Blynk.virtualWrite(V0, "Timer on");
    if ((timeToLearn - curTime) > 0){
      Serial.println(timeToLearn - curTime);
      curTime += 1;
      for (int i = 0; i < (9-((timeToLearn - curTime)/tenprecent)); ++i) {
      CircuitPlayground.setPixelColor(i, 0, 0, 0);
      }
    }
    if ((timeToLearn - curTime) == 0){
      timerFinished();
    }
    return ((timeToLearn - curTime) > 0);
}

void timerFinished(){
  Serial.println("Timer finished");
  for (int i = 0; i < 10; ++i) {
  CircuitPlayground.setPixelColor(i, 0, 0, 0);
  }
  delay(500);
  for (int i = 0; i < 10; ++i) {
  CircuitPlayground.setPixelColor(i, 255, 0, 255);
  }
  for (int i = 0; i < 5; ++i) {
  CircuitPlayground.playTone(NOTE_C4,250);
  }
  for (int i = 0; i < 10; ++i) {
  CircuitPlayground.setPixelColor(i, 0, 0, 0);
  }
}

//start timer button
BLYNK_WRITE(V4){
  timerStart = param.asInt();
}

//input of time to learn in seconds
BLYNK_WRITE(V5){
  timeToLearn = param.asInt();
  tenprecent = timeToLearn / 10;
  Serial.println(timeToLearn);
}

//////////////////////****BREAK_TIMER****//////////////////////////////////////////////////

float Z;
boolean noteStudy = false;
boolean noteBreak = false;
void onABreak(){
  Blynk.virtualWrite(V1, "---");
  Blynk.virtualWrite(V3, "---");
  Blynk.virtualWrite(V0, "Enjoy your break :)");
  Blynk.virtualWrite(V2, "Enjoy your break :)");
  if (!noteBreak){
    Blynk.notify("Get a break!");
    noteBreak = true;
  }
}

//////////////////////////////////////////////////////////////////////////

void setup() {
  CircuitPlayground.begin();
  SerialUSB.begin(9600);
  EspSerial.begin(ESP8266_BAUD);
  delay(10);
  Blynk.begin(auth, wifi, ssid, pass);
  initialize();
}


void loop() {
  Blynk.run();
  //Get a break if the cpx is turned upside down
  Z = CircuitPlayground.motionZ();
  if (Z < -5){
    onABreak();
    noteStudy = false;
  }
  else {
    if(!noteStudy){
      noteStudy = true;
      Blynk.notify("Good to have you back!");
    }
    noteBreak = false;
    studyEnv();
  }
}

//////////////////////****STUDY_ENV - MAIN_FUNCTION****////////////////////////////////////

void studyEnv(){
  //timer
  if (timerStart && isTimer == false){
    curTime = 0;
    timerStart = 0;
    isTimer = true;
    for (int i = 0; i < 10; ++i) {
      CircuitPlayground.setPixelColor(i, 0, 0, 255);
    }
  }
  if (isTimer){
    isTimer = timerOn();
  }
  else{
    //quite study env check
    soundSensor = CircuitPlayground.mic.soundPressureLevel(50);
    if (soundSensor > 64){
      if (!noteNoise){
        noteNoise = true;
        Blynk.notify("Go to a more quite place!");
      }
      Blynk.virtualWrite(V1, soundSensor);
      Blynk.virtualWrite(V0, "This place is too loud!");
    }
    else{
      noteNoise = false;
      Blynk.virtualWrite(V1, soundSensor);
      Blynk.virtualWrite(V0, "This place is quite enough!");
    }
    //lit study env check
    lightSensor = CircuitPlayground.lightSensor();
    if (lightSensor < 30){
      if (!noteDark){
        noteDark = true;
        Blynk.notify("Go to a more lit place!");
      }
      Blynk.virtualWrite(V3, lightSensor);
      Blynk.virtualWrite(V2, "This place is too dark!");
    }
    else{
      noteDark = false;
      Blynk.virtualWrite(V3, lightSensor);
      Blynk.virtualWrite(V2, "This place is lit enough!");
    }
  }
}

//////////////////////****PHONE_USED_ALARM****///////////////////////////////

void playNote(int frequency, int duration) { 
  CircuitPlayground.playTone(frequency, duration);
}

BLYNK_WRITE(V11) { 
  //acceleration force applied to axis x
  int xAxis = param[0].asFloat();  
  //acceleration force applied to axis y
  int yAxis = param[1].asFloat(); 
  //acceleration force applied to axis y
  int zAxis = param[2].asFloat();
  if (xAxis > 1 or xAxis < -1 or zAxis > 10 or zAxis < 8 or yAxis > 1 or yAxis < -1){   
    Serial.println("phone touched");     
    for (int i = 0; i < 13; ++i) {     
      CircuitPlayground.setPixelColor(i, 255, 0, 0);
      }    
    playNote(587, 275);   
    delay(100);    
    for (int i = 0; i < 13; ++i) {    
      CircuitPlayground.setPixelColor(i, 0, 0, 0); 
    }
  }    
}

////////////////////////****SPEAKER_ALARM****/////////////////////////////////////////////////////////

const uint8_t spSTOP[]              PROGMEM ={0x0E,0xD8,0x5A,0x3D,0x02,0x11,0xF0,0x00,0x40,0x00,0xBA,0x0F,0x5F,0x85,0x4F,0xBD,0x2A,0x91,0xE4,0x16,0xB7,0xBA,0xD5,0x6D,0x6E,0x33,0x3A,0xDB,
                                    0xA7,0x94,0xC7,0xAE,0xEF,0x01,0x00,0x04,0xA0,0xBD,0x39,0x06,0xE0,0x01,0x00,0x00};
const uint8_t spRIGHT[]             PROGMEM ={0xA2,0x65,0x21,0xDD,0x39,0x1C,0x8B,0x9A,0x8E,0x90,0xB0,0x74,0xAC,0x6A,0x3A,0x5C,0xA2,0x43,0x89,0x69,0xF8,0x52,0x8E,0x31,0x27,0xA9,0xD1,0x07,
                                    0xB3,0x54,0xDC,0xAF,0xDA,0xC4,0x1A,0x67,0x9D,0x77,0xEB,0x53,0x85,0x9C,0xE9,0xDC,0x71,0x6E,0x75,0xAA,0x58,0x33,0x4D,0x3A,0xEE,0xA9,0xD3,0xF0,0x64,0xCF,
                                    0xBA,0xA7,0xC9,0x45,0xC4,0xA2,0x2A,0xCD,0x2E,0xF5,0x29,0xB3,0x44,0x57,0x61,0x33,0x18,0x69,0x92,0xE9,0x16,0x00,0x40,0x01,0x5D,0x79,0x3C};
const uint8_t spNOW[]               PROGMEM ={0xA9,0x4D,0x9A,0xCD,0x9C,0xDB,0xD6,0xF6,0xD4,0xC9,0x77,0xAA,0x79,0x95,0x5B,0xDF,0xFA,0xD6,0xB7,0xB9,0xED,0xE8,0x9D,0xDF,0x66,0xF3,0xDA,0x7D,
                                    0x48,0x83,0x9C,0x15,0xE2,0x96,0x38,0x0E,0x69,0x50,0x65,0x83,0xDC,0x93,0xB8,0x51,0x86,0x1D,0x0C,0x73,0xEC,0x46,0xE9,0x7A,0xCD,0x58,0x96,0x19,0xB5,0xC9,
                                    0x54,0x61,0x37,0x6A,0xF0,0x3E,0xD2,0x85,0x64,0x3D,0x00,0x00};
const uint8_t spRETURN[]            PROGMEM ={0x00,0x60,0x12,0x15,0xC9,0x27,0x9C,0xF1,0xA8,0xBC,0x93,0xD4,0xD2,0x65,0xAB,0x4A,0x51,0x43,0x25,0x7B,0xDF,0x6A,0x54,0x3E,0x98,0x99,0x76,0x2B,
                                    0xD3,0x19,0x19,0x91,0x1A,0x4E,0x00,0x12,0xD0,0x4C,0xB8,0x07,0x14,0xE0,0x9D,0x19,0x02,0x78,0xB5,0x5C,0x91,0x4F,0x2E,0x95,0xEA,0x7A,0x27,0x2B,0x0D,0xA5,
                                    0xC9,0x8B,0x3C,0xEF,0x74,0x67,0x23,0x0F,0xDB,0x38,0x55,0x32,0xCF,0x6A,0xD4,0x61,0x3A,0x87,0x8B,0xAF,0xD6,0x84,0x6B,0x66,0x4E,0x9D,0x4A,0x1B,0x0F,0xBB,
                                    0x86,0xA4,0x09,0x5D,0x58,0x90,0x66,0xE5,0xCA,0x75,0x2C,0x58,0x9B,0xB4,0x69,0xDF,0xF9,0xEE,0x01};
const uint8_t spHOME[]              PROGMEM ={0x08,0x50,0x71,0x93,0x00,0x36,0x54,0x53,0x80,0x00,0x2A,0x6E,0x92,0x48,0xFA,0x5F,0x52,0x4D,0xD7,0xA3,0x1E,0xF7,0xB8,0x25,0x5A,0xEE,0x9B,0x81,
                                    0xFA,0x99,0xF6,0xAC,0x57,0xAD,0xF6,0x22,0xBA,0x1E,0xE0,0xF6,0xA6,0x37,0xAD,0x0D,0x81,0x23,0x03,0xCD,0xF6,0xAE,0x76,0xB5,0xCB,0x5D,0xEE,0x5C,0xA7,0x2D,
                                    0xAE,0x4B,0x38,0x35,0xAD,0x4A,0x6D,0xA1,0xD2,0x48,0x74,0xBA,0xB4,0x89,0x57,0x98,0x07};
const uint8_t spAND[]               PROGMEM ={0xA1,0xC8,0xD3,0x4C,0xBC,0x22,0xCF,0x62,0xE4,0x65,0x99,0xB3,0x4E,0x9C,0x9B,0xCD,0xAC,0x65,0x65,0x78,0x32,0x77,0xE2,0x96,0x97,0x58,0x2A,0x66,
                                    0xB3,0x5B,0x95,0x53,0xB1,0x9A,0x4D,0x69,0x75,0x2A,0x69,0xA2,0x3A,0xA7,0x35,0x31,0x97,0x91,0xD9,0xAC,0xD4,0x25,0x5F,0x85,0x2E,0xB3,0x43,0xAF,0x83,0x9A,
                                    0x9A,0xD7,0x76,0xBD,0x0A,0x5C,0x62,0xE3,0xC0,0xF5,0x32,0x70,0x89,0x77,0x2C,0xD3,0x4B,0x47,0x6D,0x3A,0xB1,0xD4,0xA0,0x2D,0xAE,0xD9,0x44,0x92,0x23,0x9F,
                                    0xC2,0x54,0x94,0x8B,0x78,0x6C,0xD0,0xA3,0x18,0x52,0x4C,0x75,0xCD,0x5A,0x0F};
const uint8_t spFINISH[]            PROGMEM ={0x02,0x48,0xCA,0x55,0x03,0x16,0xF0,0xC0,0x49,0x63,0x32,0xE3,0x8A,0x25,0x27,0xCB,0x39,0x9C,0xC2,0xDA,0xDE,0xEC,0x16,0xA9,0x8A,0x0E,0x22,0x30,
                                    0x9B,0xF4,0x6A,0x36,0xA3,0x2D,0xD6,0x49,0x2C,0x5A,0xCC,0xAE,0x75,0x49,0xB3,0x55,0x56,0x45,0x07,0xEC,0x3E,0xE1,0x01,0x0F,0x58,0x40,0x02,0x14,0x78};
const uint8_t spTEST[]              PROGMEM = {0x0E,0x98,0x6A,0xC9,0x00,0x2B,0x37,0xAF,0xA4,0x45,0x91,0xB0,0x5A,0x72,0xEA,0x9A,0x9D,0x23,0xE3,0xCD,0x6D,0x56,0x57,0x93,0x5A,0x78,0x2D,0xD9,
                                    0xE3,0x9E,0xEB,0x4E,0x77,0x02,0x6C,0x95,0x4A,0x80,0xDF,0xD2,0x39,0xA0,0x01,0x0D,0x48,0x80,0x01,0x4F,0x2B,0x53,0x00,0x14,0x70,0x45,0x9A,0x06,0x10,0x50,
                                    0x73,0xC3,0xFF,0x03};

void playAlarm(){
  CircuitPlayground.speaker.say(spSTOP);
  CircuitPlayground.speaker.say(spRIGHT);
  CircuitPlayground.speaker.say(spNOW);
  delay(1000);
  CircuitPlayground.speaker.say(spRETURN);
  CircuitPlayground.speaker.say(spHOME);
  CircuitPlayground.speaker.say(spAND);
  CircuitPlayground.speaker.say(spFINISH);
  CircuitPlayground.speaker.say(spTEST);
  delay(1000);
}

////////////////////////****STUDY_ZONE_LEFT****/////////////////////////////////////////////////////////

// Leaving your allocated 100m study zone will trigger alarm
BLYNK_WRITE(V6) {
  int pinValue = param.asInt(); 
  Serial.println("V6:"); 
  Serial.println(pinValue);  
  if (pinValue == 1){    
    playAlarm();
  }
}
